#!/usr/bin/env perl
# This scripts parses a master file in Excel.xls format and creates
# 
# This script is developed by chipster123@gmail.com 2013
# http://www.kosidossapps.com
#
# Feel free to distribute and modify code, but keep reference to its creator
# 
# The 'generateLocalesFiles.pl' script reads the file 'locale/translation_table.xls'
#
# It also generates an 'init.lua' file that pushes all the string symbols
# to be global variables to be used througout your app
# 
# files for each locale are stored in:
#    locale/<locale>.lua
#
# NOTE: the first locale in the 'translation_table.xls' file is the DEFAULT
#       locale that goes into the 'init.lua' file
# 
### Use a Spreadsheet CPAN module for parsing Excel spreadsheet
use strict;
use Spreadsheet::ParseExcel::Simple;
require Encode;
use File::Basename qw( fileparse );
use File::Path qw( make_path );
use File::Spec;

my $DEBUG = 0;

# Enable wide printing for extended characters
binmode STDOUT, ":utf8";


# Set up application constants
my $localeTopDir = "locale";
my $excel_file = $localeTopDir."/translation_table.xls";

# Open the file
my $xls = Spreadsheet::ParseExcel::Simple->read($excel_file);
# Access all sheets as an arra
my @sheets = $xls->sheets;

# Load the "phrases" sheet
my $phraseSheet;
foreach my $sheet (@sheets) {
    my $sheetname = $sheet->{sheet}->{Name};
    printf("Sheet '%s'\n",$sheetname) if $DEBUG > 0;
    if ($sheetname !~ 'phrases') { next; }
    $phraseSheet = $sheet;
}
printf("Phrase Sheet '%s'\n", $phraseSheet->{sheet}->{Name});

# Pull the contents of this "phrase" sheet
my $worksheet =  $phraseSheet->{sheet};
my ( $row_min, $row_max ) = $worksheet->row_range();
my ( $col_min, $col_max ) = $worksheet->col_range();

printf("Rows [%d,%d]\n",$row_min, $row_max) if $DEBUG > 0;
printf("Cols [%d,%d]\n",$col_min, $col_max) if $DEBUG > 0;

# Row 0 = Headings
# Row 1 .. (n-1) = strings
# Col 0 = String Constant
# Col 1 .. (n-1) = Locale

my $startCol = 3; # first colum of Locale-specific data
my $numLocales = $col_max - $startCol +1;
printf("NumLocales = %d\n", $numLocales);

my @constants;
my @locales = ();
# Prepare n-dimensional locales
for my $localeIdx (0 .. $numLocales-1) {
    printf("idx: %d\n", $localeIdx) if $DEBUG > 0;
    my @locale = ();
    push @{ $locales[$localeIdx] }, @locale;
}

# Create arrays to store the Constants and Locale Strings

# Skip the heading row and start with the content in row[1]
for my $row ( 1 .. $row_max ) {
    # first colum [0] is CONSTANT label
    my $col  = 0;
    my $cell = $worksheet->get_cell( $row, $col );

    # Skip rows with no CONSTANT
    next unless $cell; 

    my $constantLabel = $cell->value();

    # Skip rows with no CONSTANT Label
    next unless $constantLabel;
    # Store the constant
    push(@constants, $constantLabel);

    # second column [1] is optional prefix
    $col = 1;
    $cell = $worksheet->get_cell( $row, $col );
    my $prefix = "";
    if ($cell) { $prefix = $cell->value();}
    
    # third column [2] is optional postfix
    $col = 2;
    $cell = $worksheet->get_cell( $row, $col );
    my $postfix = "";
    if ($cell) { $postfix = $cell->value();}
    print "Postfix: ", $postfix, "\n" if $DEBUG > 0;

    # Locales begin in column [3]
    for my $col ( $startCol .. $col_max ) {
        
        my $localeIdx = $col - $startCol; # make localeIdx Zero based
        
        my $cell = $worksheet->get_cell( $row, $col );
        my $value = "";
        if ($cell) { $value = $cell->value() ;}
        
        #print "Row, Col    = ($row, $col)\n";
        #print "Value       = ", $value,  "\n";

        my $newString = sprintf("%s%s%s",$prefix,$value,$postfix);
        $newString =~ s/\n/\\n/g;

        # Replace all '\n' with literal '\n'
        #print "STRING      = ",$newString, "\n";
        #print "\n";

        # Store this string into the Locale's array
        push @{ $locales[$localeIdx] }, $newString;
    }

}

# Now that the file is parsed write out the locale '.lua' files
# Loop over all the Locales supported

for my $localeIdx ( 0 .. $numLocales-1 ) {
    # Create a file using the local string
    # <locale>.lua
    my $localeStr = $locales[$localeIdx][0];

    # Skip columns that have no locale string defined
    next unless $localeStr;
    my $dirname =  "locale";
    my $filename = sprintf("%s/%s.lua",$dirname, $localeStr);

    # Create the directory if not there
    if ( ! -d $dirname ) {
        printf("Creating directory '%s'\n",$dirname);
        make_path $dirname or die "Failed to create path: $dirname";
    }
    # Enable wide printing in output files:
    open (OUTFILE, '>:utf8', $filename);

    print OUTFILE "-- This file is AUTOGENERATED by 'generateLocaleFiles.pl' --\n";
    printf OUTFILE ("local l = {}\n");

    #  Loop over the constants
    for my $constIdx ( 0 .. scalar(@constants)-1 ) {
        printf OUTFILE ("l[%s] = \"%s\"\n", 
               $constants[$constIdx],
               $locales[$localeIdx][$constIdx]);
    }

    printf OUTFILE ("\nreturn l\n");
    close (OUTFILE);
    printf("Wrote file '%s'\n", $filename);

    # Special case for first locale, this is the default for the application
    # and goes into the 'init.lua' file in current directory
    if ( $localeIdx == 0 ) {
        my $filename = "init.lua";
        open (OUTFILE, '>:utf8', $filename);

        print OUTFILE "-- This file is AUTOGENERATED by 'generateLocaleFiles.pl' --\n";
        printf OUTFILE ("local l = {}\n");
        #    Loop over the constants
        for my $constIdx ( 0 .. scalar(@constants)-1 ) {
            printf OUTFILE ("l.%s = \"%s\"\n", 
                            $constants[$constIdx],
                            $locales[$localeIdx][$constIdx]);
        }
        # Special code to push variables GLOBAL
        printf OUTFILE ("--  Push the table entries into global variables\n");
        printf OUTFILE ("for k,v in pairs(l) do\n   _G[k] = v\nend\n");
        
        close (OUTFILE);
        printf("Wrote file '%s'\n", $filename);

    } # end first locale-as-default
} # end loop over locales

